// place files you want to import through the `$lib` alias in this folder.

import { building } from "$app/environment";
import { createTwitchClient } from "./twitch";

type WLEDAPI = {
    seg: Array<{ id: string; col: Array<[number, number, number]> }>;
};

const getAddr = (addr: string) => `http://${addr}/json/state`;

const buildReq = (hex: string): WLEDAPI => {
    // hex is not validated.

    const hexValues = parseInt(hex.slice(1), 16);

    const r = (hexValues >> 16) & 255;
    const g = (hexValues >> 8) & 255;
    const b = hexValues & 255;

    return {
        seg: [
            {
                id: "0",
                col: [[r, g, b]],
            },
        ],
    };
};

const preset = { ps: 5 };

const timeout = (delay: number) => new Promise((r) => setTimeout(r, delay));

export const createLedClient = (_addr: string) => {
    const addr = getAddr(_addr);

    const getState = async () => {
        return fetch(addr).then(r => r.json());
    }

    const setState = async (body: object) => {
        return fetch(addr, { method: 'POST', body: JSON.stringify(body) });
    }

    return {
        async setColor(hex: string) {
            await fetch(addr, {
                method: "POST",
                body: JSON.stringify(buildReq(hex)),
            });
        },

        async setRave(duration: number) {
            const currentState = await getState();

            await setState(preset);

            await timeout(duration);

            await setState(currentState);
        }
    };
};

export const twitchClient = createTwitchClient();

if (!building) {
    twitchClient.subscribeToFollow();
}

export const client = createLedClient("192.168.1.215");
